/**
* The MIT License (MIT)
* 
* Copyright (c) 2013 Retroswitch LLC
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
*
*/

#ifndef _PAINTCLIENT_H
#define _PAINTCLIENT_H

//
// Includes

#include "Types.h"
#include "ui_PaintClient.h"
#include "../PaintServer/src/Message.h"
#include <QtGui/QMainWindow>
#include <QLabel>
#include <QPainter>
#include <QtNetwork>

//
// Forward Declarations

class Canvas;
class QPaintEvent;
class QMouseEvent;
class QResizeEvent;

//
// PaintClient Class

class PaintClient : public QMainWindow
{
    Q_OBJECT

public:

    // --- Construction/Initialization ---

                            PaintClient( QWidget *parent = 0, Qt::WFlags flags = 0 );
                           ~PaintClient();

private slots:

    void                    OnActionConnect();
    void                    OnActionDisconnect();

    void                    OnActionClear();
    void                    OnActionLoadImage();
    void                    OnActionBorderColor( QAction* action );
    void                    OnActionBackgroundColor( QAction* action );
    void                    OnActionPenColor( QAction* action );

    void                    OnSocketConnected();
    void                    OnSocketDisconnected();
    void                    OnSocketReadReady();

    void                    OnCellChanged( uint8 x, uint8 y, uint8 color );

private:

    // --- Private Types ---

    enum State
    {
        cStateDisconnected,
        cStateConnecting,
        cStateWaitingForUpdate,
        cStateConnected,
        
        cNumStates
    };

    // --- Private Methods ---

    void                    SetState( State state );
    void                    ConnectToServer( const QString& ipAddress, uint16 port );    
    void                    HandleUpdateMessage( const MessageHeader* header, const UpdatePayload* updatePayload );
    void                    SendCommandMessage( uint8 commandId, uint8 arg0 = 0, uint8 arg1 = 0, uint8 arg2 = 0 );
    void                    SendScanlineMessage( uint8 row, uint8 length, const uint8* data );
    QActionGroup*           GenerateColorMenu( QMenu* menu, const QList<QColor>& colors, int initialSelection );

    // --- Private Members ---

    Ui::PaintClientClass    m_ui;
    QLabel*                 m_statusLabel;
    Canvas*                 m_canvas;

    State                   m_state;

    QTcpSocket*             m_tcpSocket;
    QByteArray              m_message;

};

//
// PaintClient Inline Methods

//
//

#endif // _PAINTCLIENT_H
