/**
* The MIT License (MIT)
* 
* Copyright (c) 2013 Retroswitch LLC
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
*
*/

#ifndef _TYPES_H
#define _TYPES_H

//
// Includes

#include <stddef.h>
#include <assert.h>

//
// Defines & Macros

#ifdef _WIN32
#define OS_WINDOWS
#elif defined( __linux__ )
#define OS_LINUX
#elif defined( __APPLE__ )
#define OS_OSX
#endif

//
// Type Definitions

#if defined( OS_WINDOWS )
  typedef signed __int8     int8;
  typedef signed __int16    int16;
  typedef signed __int32    int32;
  typedef unsigned __int8   uint8;
  typedef unsigned __int16  uint16;
  typedef unsigned __int32  uint32;
  typedef signed __int64    int64;
  typedef unsigned __int64  uint64;
#elif defined( OS_LINUX ) || defined( OS_OSX )
  typedef signed char       int8;
  typedef signed short      int16;
  typedef signed int        int32;
  typedef unsigned char     uint8;
  typedef unsigned short    uint16;
  typedef unsigned int      uint32;
#else
  typedef signed char       int8;
  typedef signed short      int16;
  typedef signed long       int32;
  typedef unsigned char     uint8;
  typedef unsigned short    uint16;
  typedef unsigned long     uint32;
#endif

//
//

#endif // _TYPES_H
