/**
* The MIT License (MIT)
* 
* Copyright (c) 2013 Retroswitch LLC
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
*
*/

#ifndef _DISPLAY_H
#define _DISPLAY_H

//
// Includes

#include "Types.h"

//
// Forward Declarations

//
// Functions

        // Startup display subsystem.
void    DisplayStartup();

        // Shutdown display subsystem.
void    DisplayShutdown();

        // Enable status overlay with the given status text. Each line must
        // be <= 22 characters in length (not including null terminator).
        // NOTE: No drawing (or clearing) should be performed while status
        // overlay is enabled.
void    DisplayStatusEnable( const char* line1, const char* line2 );

        // Disable status overlay.
void    DisplayStatusDisable();

        // Clear display.
void    DisplayClear();

        // Return display width/height.
uint8   DisplayGetWidth();
uint8   DisplayGetHeight();

        // Get/set border color (if supported).
uint8   DisplayGetBorderColor();
void    DisplaySetBorderColor( uint8 color );

        // Get/set background color (if supported).
uint8   DisplayGetBackgroundColor();
void    DisplaySetBackgroundColor( uint8 color );

        // Get/set cell color (occupied cells are solid if set with a valid color).
        // If color == 0xff (255), cell will be cleared, revealing the current
        // background color.
uint8   DisplayGetCell( int x, int y );
void    DisplaySetCell( int x, int y, uint8 color );

//
//

#endif // _DISPLAY_H
