/**
* The MIT License (MIT)
* 
* Copyright (c) 2013 Retroswitch LLC
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
*
*/

//
// Includes

#include "Message.h"
#include "Flyer.h"
#include <cbm.h>

//
// Functions

/******************************************************************************/
/* MessageGetHeader                                                           */
/******************************************************************************/

int8 MessageGetHeader( uint8 channel, MessageHeader* header )
{
    int bytes_available = FlyerGetBytesAvailable( channel );

    if( bytes_available < 0 )
    {
        return( -1 );
    }

    if( bytes_available >= sizeof( MessageHeader ) )
    {
        bytes_available -= sizeof( MessageHeader );

        if( cbm_read( channel, header, sizeof( MessageHeader ) ) != sizeof( MessageHeader ) )
        {
            return( -1 );
        }

        while( bytes_available < header->m_payloadLength )
        {
            bytes_available = FlyerGetBytesAvailable( channel );

            if( bytes_available < 0 )
            {
                return( -1 );
            }
        }
        
        return( 1 );
    }

    return( 0 );
}

/******************************************************************************/
/* MessagePut                                                                 */
/******************************************************************************/

int8 MessagePut( uint8 channel, MessageHeader* header, void* payload )
{
    if( cbm_write( channel, header, sizeof( MessageHeader ) ) != sizeof( MessageHeader ) )
    {
        return( -1 );
    }
    
    if( cbm_write( channel, payload, header->m_payloadLength ) != header->m_payloadLength )
    {
        return( -1 );
    }
    
    return( 0 );
}
