/**
* The MIT License (MIT)
* 
* Copyright (c) 2013 Retroswitch LLC
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
*
*/

#ifndef _MESSAGE_H
#define _MESSAGE_H

//
// Includes

#include "Types.h"

//
// Defines & Macros

#define MESSAGE_PROTOCOL 3

//
// Type Definitiosn

typedef enum tagMessageId
{
    cMessageIdConnect = 1,
    cMessageIdUpdate,
    cMessageIdCommand,
    cMessageIdScanline,
} MessageId;

typedef enum tagCommandId
{
    cCommandIdClear,
    cCommandIdSetColors, // arg[0] = border color, arg[1] = background color
    cCommandIdPlot, // arg[0] = x, arg[1] = y, arg[2] = pen color
} CommandId;

typedef struct tagMessageHeader
{
    uint8   m_id;
    uint8   m_pad;
    uint16  m_payloadLength;
} MessageHeader;

typedef struct tagConnectPayload
{
    uint8   m_protocol;
    uint8   m_reserved[3];
} ConnectPayload;

typedef struct tagUpdatePayload
{
    uint8   m_target;
    uint8   m_borderColor;
    uint8   m_backgroundColor;
    uint8   m_pad;
    /* plot data follows */
} UpdatePayload;

typedef struct tagCommandPayload
{
    uint8   m_id;
    uint8   m_arg[3];
} CommandPayload;

typedef struct tagScanlinePayload
{
    uint8   m_row;
    uint8   m_length;
    uint8   m_pad[2];
    /* scanline data follows (uint8 * m_length) */
} ScanlinePayload;

//
// Functions

        // Attempt to retrieve a full message from the Flyer. If a full message
        // is available, only the header will be read from the Flyer (but the
        // entire payload length will be available to read from the Flyer by the
        // caller).
        // Returns 0 if message header not available, 1 if message header
        // received and entire payload cached on the Flyer, or -1 if
        // disconnection/error occurred.
int8    MessageGetHeader( uint8 channel, MessageHeader* header );

        // Send a full message to the Flyer/remote client.
        // Returns 0 if successful, else -1 if disconnection/error occurred.
int8    MessagePut( uint8 channel, MessageHeader* header, void* payload );

//
//

#endif // _FLYER_H