/**
* The MIT License (MIT)
* 
* Copyright (c) 2013 Retroswitch LLC
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
*
*/

//
// Includes

#include "../Display.h"
#include <conio.h>
#include <string.h>

//
// Defines & Macros

#define DISPLAY_HEIGHT 25
#define CHARACTER_ADDR ((uint8*)0x8000)

//
// Local Data

static uint8 sDisplayWidth;
static uint8 sStatusBackup[2][80];

//
// Functions

/******************************************************************************/
/* DisplayStartup                                                             */
/******************************************************************************/

void DisplayStartup()
{
    sDisplayWidth = ((uint8*)213)[0] + 1;
    DisplayClear();
}

/******************************************************************************/
/* DisplayShutdown                                                            */
/******************************************************************************/

void DisplayShutdown()
{
    clrscr();
}

/******************************************************************************/
/* DisplayStatusEnable                                                        */
/******************************************************************************/

void DisplayStatusEnable( const char* line1, const char* line2 )
{
    uint8 i;

    for( i=0; i<sDisplayWidth; i++ )
    {
        sStatusBackup[0][i] = DisplayGetCell( i, 0 );
        sStatusBackup[1][i] = DisplayGetCell( i, 1 );
    }

    memset( CHARACTER_ADDR, 0xa0, sDisplayWidth * 2 );

    revers( 1 );
    cputsxy( 0, 0, line1 );
    cputsxy( 0, 1, line2 );
    revers( 0 );
}

/******************************************************************************/
/* DisplayStatusDisable                                                       */
/******************************************************************************/

void DisplayStatusDisable()
{
    uint8 i;

    for( i=0; i<sDisplayWidth; i++ )
    {
        DisplaySetCell( i, 0, sStatusBackup[0][i] );
        DisplaySetCell( i, 1, sStatusBackup[1][i] );
    }
}

/******************************************************************************/
/* DisplayClear                                                               */
/******************************************************************************/

void DisplayClear()
{
    memset( CHARACTER_ADDR, 0x20, sDisplayWidth * DISPLAY_HEIGHT );
}

/******************************************************************************/
/* DisplayGetWidth                                                            */
/******************************************************************************/

uint8 DisplayGetWidth()
{
    return( sDisplayWidth );
}

/******************************************************************************/
/* DisplayGetHeight                                                           */
/******************************************************************************/

uint8 DisplayGetHeight()
{
    return( DISPLAY_HEIGHT );
}

/******************************************************************************/
/* DisplayGetBorderColor                                                      */
/******************************************************************************/

uint8 DisplayGetBorderColor()
{
    return( 0 );
}

/******************************************************************************/
/* DisplaySetBorderColor                                                      */
/******************************************************************************/

void DisplaySetBorderColor( uint8 color )
{
    (void)color;
}

/******************************************************************************/
/* DisplayGetBackgroundColor                                                  */
/******************************************************************************/

uint8 DisplayGetBackgroundColor()
{
    return( 0 );
}

/******************************************************************************/
/* DisplaySetBackgroundColor                                                  */
/******************************************************************************/

void DisplaySetBackgroundColor( uint8 color )
{
    (void)color;
}

/******************************************************************************/
/* DisplayGetCell                                                             */
/******************************************************************************/

uint8 DisplayGetCell( int x, int y )
{
    int offset = y * sDisplayWidth + x;
    return( CHARACTER_ADDR[offset] == 0x20 ? 0xff : 1 );
}

/******************************************************************************/
/* DisplaySetCell                                                             */
/******************************************************************************/

void DisplaySetCell( int x, int y, uint8 color )
{
    int offset = y * sDisplayWidth + x;
    CHARACTER_ADDR[offset] = (color == 1) ? 0xa0 : 0x20;
}
